<?php

declare(strict_types=1);

use Phplrt\Compiler\Compiler;

const ROOT_DIRECTORY        = __DIR__ . '/..';
const COMPOSER_AUTOLOADER   = ROOT_DIRECTORY . '/vendor/autoload.php';
const GRAMMAR_OUTPUT        = ROOT_DIRECTORY . '/src/grammar.php';

if (!is_file(COMPOSER_AUTOLOADER)) {
    throw new \LogicException('Composer need to be installed');
}

require COMPOSER_AUTOLOADER;

if (!class_exists(Compiler::class)) {
    throw new \LogicException('Composer dev-dependencies need to be installed');
}

$compiler = new Compiler();

$compiler->load('

    // All digits sequence should be recognized as "number"
    %token number \d+
    
    // All "+" chars should be recognized as "plus"
    %token plus \+

    // All whitespace chars should be ignored
    %skip whitespace \s+

    // This means that each "expression" matches the sequence:
    //  - "number"
    //  - then optional (from 0 to infinity):
    //      - "plus"
    //      - then another "expression"
    expression -> {
        // in case of "$children" sequence is a "number()" and "expression()"
        if (count($children) === 2) {
            return new \Calculator\Node\Addition($children[0], $children[1]);
        }

        // otherwise
        return $children;
    }
      : number() (::plus:: expression())?

    number -> {
        return new \Calculator\Node\Number((int)$token->getValue()); 
    }
      : <number>
');

$assembly = $compiler->build();

file_put_contents(GRAMMAR_OUTPUT, $assembly->generate());
